/************************************************************************************/
/* file - AssessmentXMLGenerator.java                                               */
/* ----------------------------------                                               */
/* This class implements the programming logic to export assessment results to XML  */
/* It extends the thread class to allow it to run in a seperate thread of execution */
/* To properly run this class you must supply a vector which contains the assessment*/
/* IDs of the assessments you wish to export to XML                                 */
/* ----------------------------------                                               */
/* References:                                                                      */
/* http://pooh.east.asu.edu/Cet427/ClassNotes/Background/cnBackground-24.html       */
/* http://www.ibiblio.org/xml/books/xmljava/chapters/ch03s03.html                   */
/* ----------------------------------                                               */
/* NIST Self-Assessment Tool                                                        */
/* Author - Mark McLarnon                                                           */
/* Created - 02/19/02                                                               */
/* ----------------------------------                                               */
/* Change Control:                                                                  */
/* 2/28/02 - Had to add code to insert the person responsible for answering each    */
/*           question as a sub-element to the <question> node.                      */
/* 3/16/02 - Had to add code to store the results of the assessors table in the xml */
/*           before the question results.                                           */
/*         - Added code to store question that was marked as na or not              */
/************************************************************************************/
import java.util.*;
import java.util.Date;
import java.io.*;
import java.lang.*;
import java.sql.*;
import javax.swing.*;
import java.text.DecimalFormat;

class AssessmentXMLGenerator extends Thread {

    // Default constructor Simply instantiates the assessment_pool vector
    public AssessmentXMLGenerator() {
        try {
            assessment_pool = new Vector();
            this.setExportFileName("AssessmentExport" +  this.getTime() + ".xml");
            xml_writer = new PrintWriter(new BufferedWriter(new FileWriter(new File(this.getExportFileName()))));
            
            this.prgMonitor = new ProgressDialog();
            this.prgMonitor.clear();
            this.prgMonitor.setStatusBusyMessage("");
            this.prgMonitor.setVisible(false);
            
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    // Alternate constructor, takes a parameter of type vector which holds the
    // assessments that are to be exported to XML.
    public AssessmentXMLGenerator(Vector new_pool, ASSETPropertyManager props) {
        try {
            this.setAssessmentPool(new_pool);
            this.setProperties(props);
            this.setExportFileName("AssessmentExport" +  this.getTime() + ".xml");
            this.prgMonitor = new ProgressDialog();
            this.prgMonitor.clear();
            this.prgMonitor.setStatusBusyMessage("");
            this.prgMonitor.setVisible(false);
            
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void resetOutputWriter() {
        try {
            xml_writer = null;
            xml_writer = new PrintWriter(new BufferedWriter(new FileWriter(new File(this.getExportFileName()))));
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public static AssessmentXMLGenerator spawnNewXMLGenerator(Vector new_pool, ASSETPropertyManager props) {
        try {
            return new AssessmentXMLGenerator(new_pool, props);
        }catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    
    private String getTime() {
        StringBuffer time_stamp = new StringBuffer("");
        try {
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.setTime(new Date());
            
            DecimalFormat twoDigitFormat = new DecimalFormat();
	        twoDigitFormat.setMinimumIntegerDigits(2);

            int hour = calendar.get(calendar.HOUR_OF_DAY);
	        int minute = calendar.get(calendar.MINUTE);
	        int second = calendar.get(calendar.SECOND);
	        int milli_second = calendar.get(calendar.MILLISECOND);
            time_stamp.append(twoDigitFormat.format((hour)) + twoDigitFormat.format((minute)) + twoDigitFormat.format((second)));
       }catch (Exception e) {
           e.printStackTrace();
       }
       return time_stamp.toString();
    }/* end method getTime() */
    
    // This method is designed to allow the class to open up more than one 
    // connection to the database at the same time while generating the XML 
    // output. 
    private Connection connectionGenerator() {
        try {
            return DriverManager.getConnection(this.props.getProperty("sat.data.connstring"), this.props.getProperty("sat.data.username"), this.props.getProperty("sat.data.password"));
        }catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    
    private void setProperties(ASSETPropertyManager new_props) {
        this.props = new_props;    
    }
    
    public void setExportFileName(String file_name) {
        this.export_file_name = file_name;
    }
    
    private String getExportFileName() {
        return this.export_file_name;    
    }
    
    // This method returns a cross-platform new line separator.
    private String newLine() {
        return this.uniform_nl;
    }
    
    private void setAssessmentPool(Vector new_pool) {
        this.assessment_pool = new_pool;    
    }
    
    private String openXML() {
        return "<?xml version=\"1.0\"?>";    
    }
    
    private String openAssessments() {
        return "<assessments>";    
    }
    
    private String closeAssessments() {
        return "</assessments>";    
    }
    
    public void writeRawXML(String src) {        
        try {
            if (this.xml_writer != null) {
                this.xml_writer.print(src + this.newLine());
            }else {
                System.out.println("XML printer was not initialized!");
                System.exit(0);    
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void writeText(String src) {
        try {
            this.xml_writer.print(src + this.newLine());
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    // This method will print a valid XML comment to the output file. All the programmer
    // needs to do is supply the text of the comment. The method will print the special 
    // chars and the new line character
    public void printComment(String comment) {
        if (xml_writer != null) {
            xml_writer.print("<!--" + comment + "-->" + this.newLine());    
        }else {
            System.out.println("XML writer was not initialized!");    
        }
    }
    
    public void closeDocument() {
        try {
            this.xml_writer.close();  
            System.out.println("XML generation complete!");
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void displayStatusDialog(String msg) {
        try {
            JOptionPane.showMessageDialog(null, msg, "System Message:", JOptionPane.INFORMATION_MESSAGE);
        }catch (Exception e) {
            e.printStackTrace();
        }
    }	
    
    public void run() {
        CallableStatement current_assessments = null;
        CallableStatement conn_systems = null;
        CallableStatement criticalities = null;
        CallableStatement assessors = null;
        CallableStatement policy = null;
        CallableStatement critical = null;
        CallableStatement questions = null;
        ResultSet assessments = null;
        ResultSet assessor_results = null;
        ResultSet c_systems_results = null;
        ResultSet criticality_results = null;
        ResultSet policy_results = null;
        ResultSet critical_results = null;
        ResultSet question_results = null;
        String assessment = "";
        String primary_assessor_name = "";
        String component_num = "";
        int progress = 0;
        StringTokenizer parser = null;
        try {
            this.prgMonitor.setProgressMax(100);
            this.prgMonitor.clear();
            this.prgMonitor.setStatusBusyMessage("Exporting Assessment results to XML now, please hold...");
            this.prgMonitor.setVisible(true);
            this.writeRawXML(this.openXML());
            this.printComment("NIST Self-Assessment Tool Auto-generated Assessment Results");
            this.printComment("Generated on: " + (new Date()).toString());
            this.printComment("This file contains the results of " + assessment_pool.size() + " assessments");

            this.writeRawXML(this.openAssessments());
                        
            for (int i = 0; i < this.assessment_pool.size(); i++) {
                parser = new StringTokenizer((String) this.assessment_pool.elementAt(i), "^^");
                Connection conn = this.connectionGenerator();
                current_assessments =  (conn).prepareCall("{call get_single_saved_assessment(?, ?)}");
                System.out.println("assessment being exported is: " + (String) this.assessment_pool.elementAt(0)+ "^^");
                String component = (parser.nextToken());
                String date = parser.nextToken();
                System.out.println(date);
                current_assessments.setString(1, component);
                current_assessments.setString(2, date);
                assessments = current_assessments.executeQuery();

                assessments.next();                                     
                this.writeRawXML("\t<assessment>");
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());                
                this.writeRawXML("\t\t<location>");
                this.writeText("\t\t      blank");
                this.writeRawXML("\t\t</location>");
                this.writeRawXML("\t\t<system>");
                this.writeRawXML("\t\t    <name>");
                this.writeText("\t\t        " + assessments.getString("component_name"));
                this.writeRawXML("\t\t    </name>");
                this.writeRawXML("\t\t    <num>");
                component_num = assessments.getString("component_num");
                this.writeText("\t\t        " + component_num);
                this.writeRawXML("\t\t    </num>");
                this.writeRawXML("\t\t    <location>");
                this.writeText("\t\t        " +assessments.getString("component_location"));
                this.writeRawXML("\t\t    </location>");
                this.writeRawXML("\t\t    <type>");
                this.writeText("\t\t        " +assessments.getString("component_type"));
                this.writeRawXML("\t\t    </type>"); 
                this.writeRawXML("\t\t    <assessment_obj>");
                this.writeText("\t\t        " +assessments.getString("component_assessment_obj"));
                this.writeRawXML("\t\t    </assessment_obj>");                 
                this.writeRawXML("\t\t</system>");  
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                assessment = assessments.getString("assessment_id");
                this.writeRawXML("\t\t<assessmentname>");
                this.writeText("\t\t        " + assessment);
                this.writeRawXML("\t\t</assessmentname>");
                this.writeRawXML("\t\t<startdate>");
                this.writeText("\t\t        " +assessments.getString("start_timestamp"));
                this.writeRawXML("\t\t</startdate>");
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());                
                this.writeRawXML("\t\t<completed>");
                this.writeText("\t\t        " +new Integer(assessments.getInt("completed")).toString());
                this.writeRawXML("\t\t</completed>");
                this.writeRawXML("\t\t<bookmark>");
                this.writeText("\t\t        " +assessments.getString("bookmark"));
                this.writeRawXML("\t\t</bookmark>");
                primary_assessor_name = assessments.getString("primary_assessor");
                this.writeRawXML("\t\t<primary_assessor>");
                this.writeText("\t\t        " + primary_assessor_name);
                this.writeRawXML("\t\t</primary_assessor>");
                this.writeRawXML("\t\t<progress>");
                this.writeText("\t\t        " + assessments.getInt("progress"));
                this.writeRawXML("\t\t</progress>");                
                conn.close();
                
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                conn = this.connectionGenerator();
                assessors = conn.prepareCall("{call get_assessors_per_assessment(?)}");
                assessors.setString(1, assessment);
                assessor_results = assessors.executeQuery();
                
                Hashtable duplicateAssessors = new Hashtable();
                
                this.writeRawXML("\t\t<assessors>");
                while (assessor_results.next()) {
                	
                	String fullname = assessor_results.getString(  "full_name" );
                	String email    = assessor_results.getString( "email" );
                	String phone    = assessor_results.getString("phone");
                	
					duplicateAssessors.put( fullname , new Integer(1));
					                	
                    this.writeRawXML("\t\t    <assessor>");
                    this.writeRawXML("\t\t        <fullname>");
                    this.writeRawXML("\t\t            " +  fullname );
                    this.writeRawXML("\t\t        </fullname>");
                    this.writeRawXML("\t\t        <email>");
                    this.writeRawXML("\t\t            " +  email );
                    this.writeRawXML("\t\t        </email>");
                    this.writeRawXML("\t\t        <phone>");
                    this.writeRawXML("\t\t            " +  phone );
                    this.writeRawXML("\t\t        </phone>");
                    this.writeRawXML("\t\t    </assessor>");
                }
                
                assessors = conn.prepareCall("{call get_assessorreviewers_per_assessment(?)}");
                assessors.setString(1, assessment);
                assessor_results = assessors.executeQuery();
                while (assessor_results.next()) {
                	
            	    String fullname = assessor_results.getString(  "full_name" );
                	String email    = assessor_results.getString( "email" );
                	String phone    = assessor_results.getString("phone");
                
                	if ( ! duplicateAssessors.containsKey( fullname ) )
                	{
	                    this.writeRawXML("\t\t    <assessor>");
    	                this.writeRawXML("\t\t        <fullname>");
        	            this.writeRawXML("\t\t            " +  fullname );
            	        this.writeRawXML("\t\t        </fullname>");
                	    this.writeRawXML("\t\t        <email>");
                    	this.writeRawXML("\t\t            " +  email );
	                    this.writeRawXML("\t\t        </email>");
    	                this.writeRawXML("\t\t        <phone>");
        	            this.writeRawXML("\t\t            " +  phone );
            	        this.writeRawXML("\t\t        </phone>");
                	    this.writeRawXML("\t\t    </assessor>");		
                		
                	}               	                	
                }
                
                
                this.writeRawXML("\t\t</assessors>");
                
                
                
                
                conn.close();
                
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                conn = this.connectionGenerator();
                criticalities =  (conn).prepareCall("{call get_component_criticalities(?, ?)}");
                criticalities.setString(1, component_num);
                criticalities.setString(2, assessment);
                criticality_results = criticalities.executeQuery();
                
                this.writeRawXML("\t\t<criticalities>");
                while (criticality_results.next()) {
                    this.writeRawXML("\t\t<confidentiality>");
                     this.writeText("\t\t        " + criticality_results.getString("confidentiality"));
                    this.writeRawXML("\t\t</confidentiality>");
                    this.writeRawXML("\t\t<integrity>");
                     this.writeText("\t\t        " + criticality_results.getString("integrity"));
                    this.writeRawXML("\t\t</integrity>");
                    this.writeRawXML("\t\t<availability>");
                     this.writeText("\t\t        " + criticality_results.getString("availability"));
                    this.writeRawXML("\t\t</availability>");
                }
                this.writeRawXML("\t\t</criticalities>");
                conn.close();
                
                progress += 10;                
                conn = this.connectionGenerator();
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                conn_systems =  (conn).prepareCall("{call get_connected_systems(?)}");
                conn_systems.setString(1, component_num);
                c_systems_results = conn_systems.executeQuery();
                
                this.writeRawXML("\t\t<connectedsystems>");
                while (c_systems_results.next()) {
                    this.writeRawXML("\t\t<component>");
                    this.writeRawXML("\t\t    <name>");
                    this.writeText("\t\t        " + c_systems_results.getString("conn_component"));
                    this.writeRawXML("\t\t    </name>");                    
                    this.writeRawXML("\t\t    <bce>");
                    this.writeText("\t\t        " + c_systems_results.getInt("bce"));
                    this.writeRawXML("\t\t    </bce>");
                    this.writeRawXML("\t\t    <pain>");
                    this.writeText("\t\t        " + c_systems_results.getString("planned_action"));
                    this.writeRawXML("\t\t    </pain>");                    
                    this.writeRawXML("\t\t</component>");
                }
                this.writeRawXML("\t\t</connectedsystems>");
                conn.close();
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                conn = this.connectionGenerator();
                policy =  (conn).prepareCall("{call get_policy_results(?)}");
                policy.setString(1, assessment);
                policy_results = policy.executeQuery();   
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());                
                this.writeRawXML("\t\t<policy>");
                while (policy_results.next()) {
                    this.writeRawXML("\t\t<objective>");
                    this.writeRawXML("\t\t    <number>");
                    this.writeText("\t\t        " + new Integer(policy_results.getInt("major_section")).toString() + "." + new Integer(policy_results.getInt("minor_section")).toString() + "." + new Integer(policy_results.getInt("question_number")).toString());
                    this.writeRawXML("\t\t    </number>");
                    this.writeRawXML("\t\t    <state>");
                    this.writeText("\t\t        " + policy_results.getInt("level1"));
                    this.writeRawXML("\t\t    </state>");
                    this.writeRawXML("\t\t</objective>");
                }
                this.writeRawXML("\t\t</policy>");
                conn.close();
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                conn = this.connectionGenerator();
                critical =  (conn).prepareCall("{call get_critical_results(?)}");
                critical.setString(1, assessment);
                critical_results = critical.executeQuery();                 
                this.writeRawXML("\t\t<critical>");
                while (critical_results.next()) {
                    this.writeRawXML("\t\t<element>");
                    this.writeRawXML("\t\t    <number>");
                    this.writeText("\t\t        " + new Integer(critical_results.getInt("major_section")).toString() + "." + new Integer(critical_results.getInt("minor_section")).toString() + "." + new Integer(critical_results.getInt("question_number")).toString());
                    this.writeRawXML("\t\t    </number>");                  
                    this.writeRawXML("\t\t    <policy>");
                    this.writeText("\t\t        " + critical_results.getInt("level1"));
                    this.writeRawXML("\t\t    </policy>");
                    this.writeRawXML("\t\t    <procedures>");
                    this.writeText("\t\t        " + critical_results.getInt("level2"));
                    this.writeRawXML("\t\t    </procedures>");
                    this.writeRawXML("\t\t    <implemented>");
                    this.writeText("\t\t        " + critical_results.getInt("level3"));
                    this.writeRawXML("\t\t    </implemented>");                    
                    this.writeRawXML("\t\t    <tested>");
                    this.writeText("\t\t        " + critical_results.getInt("level4"));
                    this.writeRawXML("\t\t    </tested>");
                    this.writeRawXML("\t\t    <integrated>");
                    this.writeText("\t\t        " + critical_results.getInt("level5"));
                    this.writeRawXML("\t\t    </integrated>");
                    this.writeRawXML("\t\t    <rbdm>");
                    this.writeText("\t\t        " + critical_results.getInt("rbdm"));
                    this.writeRawXML("\t\t    </rbdm>"); 
                    this.writeRawXML("\t\t    <na>");
                    this.writeText("\t\t        " + critical_results.getInt("na"));
                    this.writeRawXML("\t\t    </na>");
                    this.writeRawXML("\t\t</element>");
                }
                this.writeRawXML("\t\t</critical>");
                conn.close();
                progress += 10;
                
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                conn = this.connectionGenerator();
                questions =  (conn).prepareCall("{call get_question_results(?)}");
                questions.setString(1, assessment);
                question_results = questions.executeQuery();                 
                this.writeRawXML("\t\t<questions>");
                while (question_results.next()) {
                    this.writeRawXML("\t\t<question>");
                    this.writeRawXML("\t\t    <number>");
                    this.writeText("\t\t        " + new Integer(question_results.getInt("major_section")).toString() + "." + new Integer(question_results.getInt("minor_section")).toString() + "." + new Integer(question_results.getInt("question_number")).toString());
                    this.writeRawXML("\t\t    </number>");                
                    this.writeRawXML("\t\t    <policy>");
                    this.writeText("\t\t        " + question_results.getInt("level1"));
                    this.writeRawXML("\t\t    </policy>");
                    this.writeRawXML("\t\t    <procedures>");
                    this.writeText("\t\t        " + question_results.getInt("level2"));
                    this.writeRawXML("\t\t    </procedures>");
                    this.writeRawXML("\t\t    <implemented>");
                    this.writeText("\t\t        " + question_results.getInt("level3"));
                    this.writeRawXML("\t\t    </implemented>");                    
                    this.writeRawXML("\t\t    <tested>");
                    this.writeText("\t\t        " + question_results.getInt("level4"));
                    this.writeRawXML("\t\t    </tested>");
                    this.writeRawXML("\t\t    <integrated>");
                    this.writeText("\t\t        " + question_results.getInt("level5"));
                    this.writeRawXML("\t\t    </integrated>");
                    this.writeRawXML("\t\t    <rbdm>");
                    this.writeText("\t\t        " + question_results.getInt("rbdm"));
                    this.writeRawXML("\t\t    </rbdm>"); 
                    this.writeRawXML("\t\t    <cmnts>");
                    this.writeText("\t\t        " + (question_results.getString("cmnts")).trim());
                    this.writeRawXML("\t\t    </cmnts>");
                    this.writeRawXML("\t\t    <ar>");
                    this.writeText("\t\t        " + question_results.getInt("ar"));
                    this.writeRawXML("\t\t    </ar>");
                    this.writeRawXML("\t\t    <na>");
                    this.writeText("\t\t        " + question_results.getInt("na"));
                    this.writeRawXML("\t\t    </na>");
                    this.writeRawXML("\t\t    <reviewer>");
                    this.writeText("\t\t        " + (question_results.getString("reviewer")).trim());
                    this.writeRawXML("\t\t    </reviewer>");
                    this.writeRawXML("\t\t    <completed>");
                    this.writeText("\t\t        " + (question_results.getInt("completed")));
                    this.writeRawXML("\t\t    </completed>");                    
                    this.writeRawXML("\t\t    <answeredby>");
                    this.writeText("\t\t        " + question_results.getString("full_name"));
                    this.writeRawXML("\t\t    </answeredby>");

                    this.writeRawXML("\t\t</question>");
                }
                this.writeRawXML("\t\t</questions>");
                conn.close();
                progress += 10;
                this.prgMonitor.setNormalizedProgress((progress) / this.assessment_pool.size());
                this.writeRawXML("\t</assessment>");
            }
            this.writeRawXML(this.closeAssessments());
            this.closeDocument();
            this.prgMonitor.setVisible(false);
            this.displayStatusDialog("XML Assessment export complete!");
        }catch (Exception e) {
            
            e.printStackTrace();
        }
    }/* end method run() */
        
    PrintWriter xml_writer = null;
    
    Vector assessment_pool = null;
    private String export_file_name = "AssessmentExport.xml";
    
    ASSETPropertyManager props = null;
    String uniform_nl = System.getProperty("line.separator");
    ProgressDialog prgMonitor = null;
}/* end class AssessmentXMLGenerator() */